interface PubSubDeferredPromise<T> {
    promise: Promise<void>;
    resolve: () => void;
    isDone: boolean;
    values: Array<T>;
}
export interface PubSubEngine {
    publish<TPayload>(topic: string, payload: TPayload): void | Promise<void>;
    subscribe<TPayload>(...topics: [string, ...string[]]): AsyncGenerator<TPayload>;
    unsubscribe(...iterators: Array<AsyncGenerator<unknown>>): void | Promise<void>;
    close(): void | Promise<void>;
}
declare class BaseInMemoryPubSub {
    withFilter: typeof withFilter;
    protected readonly ValuePromiseMap: WeakMap<AsyncGenerator<unknown>, PubSubDeferredPromise<any>>;
    protected readonly TopicIteratorMap: Map<string, Set<AsyncGenerator<unknown>>>;
    protected publishTopic<Topic extends string, Payload>(topic: Topic, payload: Payload): void;
    protected subscribeTopics<Topic extends string, Payload>(...topics: [Topic, ...Topic[]]): AsyncGenerator<Payload>;
    protected unsubscribeIterators(...iterators: AsyncGenerator<unknown>[]): void;
    protected closeIterators(): void;
}
export declare class InMemoryPubSub extends BaseInMemoryPubSub implements PubSubEngine {
    publish<TPayload>(topic: string, payload: TPayload): void;
    subscribe<TPayload = any>(...topics: [string, ...string[]]): AsyncGenerator<TPayload>;
    unsubscribe(...iterators: AsyncGenerator<unknown>[]): void;
    close(): void;
}
export interface StrictPubSubEngine<Topics extends Record<string, unknown>> {
    publish<Topic extends Exclude<keyof Topics, number | symbol>>(topic: Topic, payload: Topics[Topic]): void | Promise<void>;
    subscribe<Topic extends Exclude<keyof Topics, number | symbol>>(...topics: [Topic, ...Topic[]]): AsyncGenerator<Topics[Topic]>;
    unsubscribe(...iterators: Array<AsyncGenerator<unknown>>): void | Promise<void>;
    close(): void | Promise<void>;
}
export declare class StrictInMemoryPubSub<Topics extends Record<string, unknown>> extends BaseInMemoryPubSub implements StrictPubSubEngine<Topics> {
    publish<Topic extends Exclude<keyof Topics, number | symbol>>(topic: Topic, payload: Topics[Topic]): void;
    subscribe<Topic extends Exclude<keyof Topics, number | symbol>>(...topics: [Topic, ...Topic[]]): AsyncGenerator<Topics[Topic], any, unknown>;
    unsubscribe(...iterators: Array<AsyncGenerator<unknown>>): void;
    close(): void;
}
export declare function withFilter<TData, TFilteredData extends TData>(iterator: AsyncGenerator<TData> | Promise<AsyncGenerator<TData>>, filter: (data: TData) => data is TFilteredData): AsyncGenerator<TFilteredData>;
export declare function withFilter<TData>(iterator: AsyncGenerator<TData> | Promise<AsyncGenerator<TData>>, filter: (data: TData) => boolean): AsyncGenerator<TData>;
export {};
