'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function pubsubDeferredPromise() {
  let resolve;
  const promise = new Promise((resolveFn) => {
    resolve = resolveFn;
  });
  return {
    promise,
    resolve,
    values: [],
    isDone: false
  };
}
class BaseInMemoryPubSub {
  constructor() {
    this.withFilter = withFilter;
    this.ValuePromiseMap = /* @__PURE__ */ new WeakMap();
    this.TopicIteratorMap = /* @__PURE__ */ new Map();
  }
  publishTopic(topic, payload) {
    const { TopicIteratorMap, ValuePromiseMap } = this;
    const iteratorSet = TopicIteratorMap.get(topic);
    if (iteratorSet == null)
      return;
    for (const iterator of iteratorSet) {
      const valuePromise = ValuePromiseMap.get(iterator);
      if (valuePromise == null)
        continue;
      valuePromise.values.push(payload);
      valuePromise.resolve();
    }
  }
  subscribeTopics(...topics) {
    const { ValuePromiseMap, TopicIteratorMap } = this;
    const iterator = generator();
    const iteratorSets = topics.map((topic) => {
      let iteratorSet = TopicIteratorMap.get(topic);
      if (iteratorSet == null) {
        iteratorSet = /* @__PURE__ */ new Set();
        TopicIteratorMap.set(topic, iteratorSet);
      }
      iteratorSet.add(iterator);
      return iteratorSet;
    });
    let valuePromise = topics.length ? pubsubDeferredPromise() : null;
    if (valuePromise)
      ValuePromiseMap.set(iterator, valuePromise);
    async function* generator() {
      while (valuePromise != null) {
        await valuePromise.promise;
        for (const value of valuePromise.values)
          yield value;
        if (valuePromise.isDone) {
          valuePromise = null;
          ValuePromiseMap.delete(iterator);
        } else {
          valuePromise = pubsubDeferredPromise();
          ValuePromiseMap.set(iterator, valuePromise);
        }
      }
      for (const iteratorSet of iteratorSets) {
        iteratorSet.delete(iterator);
      }
    }
    return iterator;
  }
  unsubscribeIterators(...iterators) {
    const { ValuePromiseMap } = this;
    for (const iterator of iterators) {
      const valuePromise = ValuePromiseMap.get(iterator);
      if (valuePromise == null)
        continue;
      valuePromise.resolve();
      valuePromise.isDone = true;
      ValuePromiseMap.delete(iterator);
    }
  }
  closeIterators() {
    const { TopicIteratorMap, ValuePromiseMap } = this;
    for (const [topic, iteratorSet] of TopicIteratorMap.entries()) {
      for (const iterator of iteratorSet) {
        const valuePromise = ValuePromiseMap.get(iterator);
        if (valuePromise == null)
          continue;
        valuePromise.resolve();
        valuePromise.isDone = true;
        ValuePromiseMap.delete(iterator);
      }
      iteratorSet.clear();
      TopicIteratorMap.delete(topic);
    }
  }
}
class InMemoryPubSub extends BaseInMemoryPubSub {
  publish(topic, payload) {
    return this.publishTopic(topic, payload);
  }
  subscribe(...topics) {
    return this.subscribeTopics(...topics);
  }
  unsubscribe(...iterators) {
    return this.unsubscribeIterators(...iterators);
  }
  close() {
    return this.closeIterators();
  }
}
class StrictInMemoryPubSub extends BaseInMemoryPubSub {
  publish(topic, payload) {
    return this.publishTopic(topic, payload);
  }
  subscribe(...topics) {
    return this.subscribeTopics(...topics);
  }
  unsubscribe(...iterators) {
    return this.unsubscribeIterators(...iterators);
  }
  close() {
    return this.closeIterators();
  }
}
async function* withFilter(iterator, filter) {
  for await (const value of await iterator) {
    if (filter(value))
      yield value;
  }
}

exports.InMemoryPubSub = InMemoryPubSub;
exports.StrictInMemoryPubSub = StrictInMemoryPubSub;
exports.withFilter = withFilter;
